@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: pipecmd.c 1143 2007-10-23 23:34:58Z grondo $
+ *  $Id$
  *****************************************************************************
  *  Copyright (C) 2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -93,11 +93,10 @@ static void pipe_info_destroy (struct pipe_info_struct *e)
 static char * pipecmd_format_arg (pipecmd_t e, const char *arg)
 {
     char buf [64];
-    const char *q, *p;
+    const char *p;
     char *str = NULL;
 
-    q = p = arg;
-
+    p = arg;
     while (*p != '\0') {
         if (*p == '%') {
             p++;
