@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: dsh.c 1151 2008-01-29 17:45:54Z grondo $
+ *  $Id$
  *****************************************************************************
  *  Copyright (C) 2001-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -263,8 +263,8 @@ static void _fwd_signal(int signum)
 
     dsh_mutex_lock(&thd_mutex);
     for (i = 0; t[i].host != NULL; i++) {
-        if ((t[i].state == DSH_READING)) 
-			rcmd_signal(t[i].rcmd, signum);
+        if (t[i].state == DSH_READING)
+            rcmd_signal(t[i].rcmd, signum);
     }
     dsh_mutex_unlock(&thd_mutex);
 
@@ -522,8 +522,8 @@ static int _extract_rc(char *buf)
 
 static int _do_output (int fd, cbuf_t cb, out_f outf, bool read_rc, thd_t *t)
 {
-    char buf[8192];
-    int rc = 0;
+    char c;
+    int n, rc;
     int dropped = 0;
 
     if ((rc = cbuf_write_from_fd (cb, fd, -1, &dropped)) < 0) {
@@ -533,22 +533,47 @@ static int _do_output (int fd, cbuf_t cb, out_f outf, bool read_rc, thd_t *t)
         return (-1);
     } 
 
-    while (cbuf_read_line (cb, buf, 8192, 1) > 0) {
+    /*
+     *  Use cbuf_peek_line with a single character buffer in order to
+     *   get the buffer size needed for the next line (if any).
+     */
+    while ((n = cbuf_peek_line (cb, &c, 1, 1))) {
+        char *buf;
 
-        if (read_rc)
-            t->rc = _extract_rc (buf);
-
-        if (strlen (buf) > 0) {
-            if (t->labels)
-                outf ("%S: %s", t->host, buf);
-            else
-                outf ("%s", buf);
-            fflush (NULL);
+        if (n < 0) {
+            err ("%p: %S: Failed to peek line: %m\n", t->host);
+            break;
         }
+
+        /*
+         *  Allocate enough space for line plus NUL character,
+         *   then actually read line data into buffer:
+         */
+        buf = Malloc (n + 1);
+        if ((n = cbuf_read (cb, buf, n))) {
+            if (n < 0) {
+                err ("%p: %S: Failed to read line from buffer: %m\n", t->host);
+                break;
+            }
+            if (read_rc)
+                t->rc = _extract_rc (buf);
+            if (strlen (buf) > 0) {
+                /*
+                 *  We are careful to use a single call to write the line
+                 *   to the output stream to avoid interleaved lines of
+                 *   output.
+                 */
+                if (t->labels)
+                    outf ("%S: %s", t->host, buf);
+                else
+                    outf ("%s", buf);
+                fflush (NULL);
+            }
+        }
+        Free ((void **)&buf);
     }
 
     return (rc);
-
 }
 
 static void _flush_output (cbuf_t cb, out_f outf, thd_t *t)
@@ -622,7 +647,7 @@ static void *_rsh_thread(void *args)
     if (a->rcmd->opts->resolve_hosts)
         _gethost(a->host, a->addr);
 #endif
-    _xsignal (SIGPIPE, SIG_BLOCK);
+    _xsignal (SIGPIPE, SIG_IGN);
 
     /* establish the connection */
     dsh_mutex_lock(&thd_mutex);
@@ -841,10 +866,13 @@ static int _thd_init (thd_t *th, opt_t *opt, List pcp_infiles, int i)
     th->pcp_progname = opt->progname;
     th->outfile_name = opt->outfile_name;
     th->kill_on_fail = opt->kill_on_fail;
-    th->outbuf = cbuf_create (64, 8192);
-    th->errbuf = cbuf_create (64, 8192);
+    th->outbuf = cbuf_create (64, 131072);
+    th->errbuf = cbuf_create (64, 131072);
 
-    th->rcmd = rcmd_create (th->host);
+    if (!(th->rcmd = rcmd_create (th->host))) {
+        th->state = DSH_CANCELED;
+        return (-1);
+    }
 
 #if	!HAVE_MTSAFE_GETHOSTBYNAME
     /* if MT-safe, do it in parallel in rsh/rcp threads */
@@ -1020,7 +1048,7 @@ int dsh(opt_t * opt)
             exit(1);
         }
 
-        xstrcat(&cmd, opt->path_progname); 
+        xstrcat(&cmd, opt->remote_program_path);
         if (opt->recursive)
             xstrcat(&cmd, " -r");
         if (opt->preserve)
@@ -1038,7 +1066,7 @@ int dsh(opt_t * opt)
         char *filename;
         ListIterator i;
 
-        xstrcat(&cmd, opt->path_progname); 
+        xstrcat(&cmd, opt->remote_program_path);
 
         if (opt->recursive)
             xstrcat(&cmd, " -r");
