@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: main.c 1102 2006-12-07 23:44:25Z grondo $
+ *  $Id$
  *****************************************************************************
  *  Copyright (C) 2001-2006 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -68,6 +68,8 @@ int main(int argc, char *argv[])
 {
     opt_t opt;
     int retval = 0;
+    const char *m;
+
 
     /*
      * Initialize.
@@ -85,17 +87,34 @@ int main(int argc, char *argv[])
      */
     opt_default(&opt, argv[0]);
 
+    /*
+     * Override defaults with environment
+     */
+    opt_env(&opt);
+
+    /*
+     * Process any options that need to be handled early:
+     */
+    opt_args_early(&opt, argc, argv);
+
     /*
      *  Load static or dynamic pdsh modules
      */
     mod_init();
-    if (mod_load_modules(pdsh_module_dir, &opt) < 0)
+    /*
+     *  Allow module directory to be overridden, but not when
+     *   running as root or setuid. (This is mainly for testing...)
+     */
+    if (!(m = getenv ("PDSH_MODULE_DIR")) ||
+          getuid() == 0 ||
+          getuid() != geteuid())
+        m = pdsh_module_dir;
+    if (mod_load_modules(m, &opt) < 0)
         errx("%p: Couldn't load any pdsh modules\n");
 
     /*
      * Handle options.
      */
-    opt_env(&opt);              /* override defaults with env variables */
     opt_args(&opt, argc, argv); /* override with command line           */
 
     if (opt_verify(&opt)) {     /* verify options, print errors         */
@@ -342,14 +361,18 @@ static void _interactive_dsh(opt_t * opt)
  */
 static void _shell(uid_t uid, char *cmd)
 {
+    int ret;
     pid_t pid;
 
     switch (pid = fork()) {
     case -1:
         errx("%p: fork: %m\n");
     case 0:
-        setuid(uid);
-        system(cmd);
+        if (setuid(uid) < 0)
+            errx ("%p: setuid: %m\n");
+        ret = system(cmd);
+        if (ret)
+            err ("%p: exited with status %d\n", ret);
         exit(0);
     default:
         waitpid(pid, NULL, 0);
